/********************************************************************************************/
/*******                                                                              *******/
/*******  HeavyMole                                                                   *******/
/*******  Copyright (C) 2000  L. Antoine <antoinem@linuxmail.org>                     *******/
/*******                      C. Van brussel <kickvb@linuxmail.org>                   *******/
/*******                                                                              *******/
/*******  This program is free software; you can redistribute it and/or               *******/
/*******  modify it under the terms of the GNU General Public License                 *******/
/*******  as published by the Free Software Foundation; either version 2              *******/
/*******  of the License, or (at your option) any later version.                      *******/
/*******                                                                              *******/
/*******  This program is distributed in the hope that it will be useful,             *******/
/*******  but WITHOUT ANY WARRANTY; without even the implied warranty of              *******/
/*******  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *******/
/*******  GNU General Public License for more details.                                *******/
/*******                                                                              *******/
/*******  You should have received a copy of the GNU General Public License           *******/
/*******  along with this program; if not, write to the Free Software                 *******/
/*******  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *******/
/*******                                                                              *******/
/********************************************************************************************/

#include <assert.h>

#include "ProgressCellRenderer.hh"

ProgressCellRenderer::ProgressCellRenderer() :
  Glib::ObjectBase(typeid(ProgressCellRenderer)),
  Gtk::CellRenderer(),
  property_progress_value_ (*this, "progress_value", 0),
  property_progress_pixmap_ (*this, "progress_pixmap")
{
  property_mode() = Gtk::CELL_RENDERER_MODE_INERT;
  property_xpad() = 0;
  property_ypad() = 0;
}

ProgressCellRenderer::~ProgressCellRenderer() {
}

Glib::PropertyProxy_Base ProgressCellRenderer::_property_renderable() {
  return property_progress_value();
}

Glib::PropertyProxy<int> ProgressCellRenderer::property_progress_value() {
  return property_progress_value_.get_proxy();
}

Glib::PropertyProxy< Glib::RefPtr<Gdk::Pixbuf> > ProgressCellRenderer::property_progress_pixmap() {
  return property_progress_pixmap_.get_proxy();
}

void ProgressCellRenderer::get_size_vfunc(Gtk::Widget& widget, const Gdk::Rectangle& cell_area, int& x_offset, int& y_offset, int& width, int& height) {
  Glib::RefPtr<Gdk::Pixbuf> progress_pixmap = property_progress_pixmap().get_value();
  width = progress_pixmap->get_width();
  height = progress_pixmap->get_height();
}

void ProgressCellRenderer::render_vfunc(const Glib::RefPtr<Gdk::Window>& window, Gtk::Widget& widget, const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, const Gdk::Rectangle& expose_area, Gtk::CellRendererState flags) {
  // get & check renderer state
  int progress_value = property_progress_value().get_value();
  assert(progress_value >= 0 && progress_value <= 100);
  Glib::RefPtr<Gdk::Pixbuf> progress_pixmap = property_progress_pixmap().get_value();
  assert(!progress_pixmap.is_null());

  // compute start position of image
  int start_x = cell_area.get_x() + (cell_area.get_width() - progress_pixmap->get_width())/ 2;
  int start_y = cell_area.get_y() + 1;

  // find current state
  Gtk::StateType state = Gtk::STATE_NORMAL;
  if ((flags & Gtk::CELL_RENDERER_SELECTED) != 0)
    state = (widget.has_focus()) ? Gtk::STATE_SELECTED : Gtk::STATE_ACTIVE;

  // get current GC's
  Glib::RefPtr<Gdk::GC> bg_gc = widget.get_style()->get_white_gc();
  // this line (or whatever allocating a second Glib::RefPtr<Gdk::GC>) can create a segfault under win32
  Glib::RefPtr<Gdk::GC> text_gc = widget.get_style()->get_text_gc(state);

  // draw foreground
  window->draw_pixbuf(bg_gc, progress_pixmap, 0, 0, start_x, start_y, progress_value, progress_pixmap->get_height(), Gdk::RGB_DITHER_NORMAL, 0, 0);    

  // prepare pango layout with percentage text
  Glib::RefPtr<Pango::Layout> layout = Pango::Layout::create(widget.get_pango_context());
  char pc[16];
  sprintf(pc, "%d%%", progress_value);
  layout->set_text(pc);
  layout->set_font_description(widget.get_style()->get_font());

  // compute text position
  int layout_w, layout_h;
  // following values are the size in pixels of the text and should be read from the layout object
  layout_w = 20;
  layout_h = 13;
  int txt_x = (progress_pixmap->get_width() - layout_w) / 2.0;
  int txt_y = (progress_pixmap->get_height() - layout_h) / 2.0;

  // draw text
  window->draw_layout(text_gc, start_x + txt_x, start_y + txt_y, layout);
}
