#include <gtkmm/main.h>
#include <string>

#include "ProgressCellRenderer.hh"

class TestTreeview : public Gtk::TreeView
{
public:

  TestTreeview() :
    value(0),
    ascend(true)
  {
    Glib::RefPtr<Gtk::ListStore> liststore = Gtk::ListStore::create(m_Columns);
    set_model(liststore);

    std::string filename = "progress1.xpm";
    try {pixbuf1 = Gdk::Pixbuf::create_from_file(filename);}
    catch (Glib::FileError e) { fprintf(stderr, "No files found in %s\n", filename.c_str()); exit(1); }
    catch (Gdk::PixbufError e) { fprintf(stderr, "Invalid image file in %s\n", filename.c_str()); exit(1); }

    filename = "progress2.xpm";
    try {pixbuf2 = Gdk::Pixbuf::create_from_file(filename);}
    catch (Glib::FileError e) { fprintf(stderr, "No files found in %s\n", filename.c_str()); exit(1); }
    catch (Gdk::PixbufError e) { fprintf(stderr, "Invalid image file in %s\n", filename.c_str()); exit(1); }

    append_column("CellRendererText", m_Columns.m_col_progress_text);

    ProgressCellRenderer *renderer = manage(new ProgressCellRenderer());
    Gtk::TreeViewColumn *col = manage(new Gtk::TreeViewColumn("ProgressCellRenderer", *renderer));
    append_column(*col);
    col->add_attribute(renderer->property_progress_value(), m_Columns.m_col_progress_value);
    col->add_attribute(renderer->property_progress_pixmap(), m_Columns.m_col_progress_pixmap);

    row1 = *liststore->append();
    row1[m_Columns.m_col_progress_text] = "abra";
    row1[m_Columns.m_col_progress_value] = 0;
    row1[m_Columns.m_col_progress_pixmap] = pixbuf1;

    row2 = *liststore->append();
    row2[m_Columns.m_col_progress_text] = "cada";
    row2[m_Columns.m_col_progress_value] = 33;
    row2[m_Columns.m_col_progress_pixmap] = pixbuf2;

    row3 = *liststore->append();
    row3[m_Columns.m_col_progress_text] = "boum!";
    row3[m_Columns.m_col_progress_value] = 66;
    row3[m_Columns.m_col_progress_pixmap] = pixbuf1;

    Glib::signal_timeout().connect(SigC::slot(*this, &TestTreeview::update_progress), 50);
  }

  bool update_progress() {
    if (ascend && value == 100) {
      ascend = !ascend;
      row1[m_Columns.m_col_progress_pixmap] = pixbuf2;
      row2[m_Columns.m_col_progress_pixmap] = pixbuf1;
      row3[m_Columns.m_col_progress_pixmap] = pixbuf2;
    }

    if (!ascend && value == 0) {
      ascend = !ascend;
      row1[m_Columns.m_col_progress_pixmap] = pixbuf1;
      row2[m_Columns.m_col_progress_pixmap] = pixbuf2;
      row3[m_Columns.m_col_progress_pixmap] = pixbuf1;
    }

    if (ascend) value++;
    else value--;

    row1[m_Columns.m_col_progress_value] = value;
    row2[m_Columns.m_col_progress_value] = (value + 33) % 100;
    row3[m_Columns.m_col_progress_value] = (value + 66) % 100;
  }  

  class ModelColumns : public Gtk::TreeModel::ColumnRecord {
  public:
    ModelColumns() {
      add(m_col_progress_text);
      add(m_col_progress_value);
      add(m_col_progress_pixmap);
    }
    Gtk::TreeModelColumn<std::string> m_col_progress_text;
    Gtk::TreeModelColumn<int> m_col_progress_value;
    Gtk::TreeModelColumn< Glib::RefPtr<Gdk::Pixbuf> > m_col_progress_pixmap;
  };

  ModelColumns m_Columns;

  Glib::RefPtr<Gdk::Pixbuf> pixbuf1, pixbuf2;
  Gtk::TreeRow row1, row2, row3;
  int value;
  bool ascend;
};

int main(int argc, char *argv[])
{
  Gtk::Main kit(argc, argv);

  TestTreeview tv;
  Gtk::Window window;
  Gtk::ScrolledWindow win;
  win.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  win.add(tv);
  window.add(win);
  window.set_default_size(200, 100);
  window.show_all();
  Gtk::Main::run();

  return 0;
}
